//===========================================================================
/*
    This file is part of the CHAI 3D visualization and haptics libraries.
    Copyright (C) 2003-2004 by CHAI 3D. All rights reserved.

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License("GPL") version 2
    as published by the Free Software Foundation.

    For using the CHAI 3D libraries with software that can not be combined
    with the GNU GPL, and for taking advantage of the additional benefits
    of our support services, please contact CHAI 3D about acquiring a
    Professional Edition License.

    \author:    <http://www.chai3d.org>
    \author:    Francois Conti
    \version    1.1
    \date       01/2006
*/
//===========================================================================

/*
							Modified by Marras Ioannis (07_08_07)
							imarras@aiia.csd,auth,gr

							Enterface Workshop, Instabul, Turkey 2007
*/

//---------------------------------------------------------------------------
#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// Different compilers like slightly different GLUT's 
#ifdef _MSVC
#include "../../../external/OpenGL/msvc6/glut.h"
#else
  #ifdef _POSIX
    #include <GL/glut.h>
  #else
    #include "../../../external/OpenGL/bbcp6/glut.h"
  #endif
#endif

//---------------------------------------------------------------------------
#include "CCamera.h"
#include "CLight.h"
#include "CWorld.h"
#include "CMesh.h"
#include "CTriangle.h"
#include "CVertex.h"
#include "CMaterial.h"
#include "CTexture2D.h"
#include "CMatrix3d.h"
#include "CVector3d.h"
#include "CPrecisionClock.h"
#include "CPrecisionTimer.h"
#include "CMeta3dofPointer.h"
#include "CShapeTorus.h"
#include "CShapeSphere.h"
#include "CBitmap.h"
#include "CLabelPanel.h"
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// QT-RELATED STUFF
#include <QApplication>
#include "Simple.h"
#include "SimpleQVTK.h"
#include "ContainerWidget.h"
#include "vavHapticMessenger.h"

Simple *GlWidget;
vavHapticMessenger *HapticMessenger = NULL;

// QT-RELATED STUFF
//---------------------------------------------------------------------------
#define USE_PHANTOM_DIRECT_IO 1

// the world in which we will create our environment
cWorld* world;

// the camera which is used view the environment in a window
cCamera* camera;

// a light source
cLight *light;

cMesh *CoorSystem,*TempMesh;

// a simple torus object
cShapeTorus* TorusObject;

// a simple sphere shape
cShapeSphere* ShapeSphere_Delete_Object,*ShapeSphere_Add_Object,*Increase_ROI_Size_Object,*Decrease_ROI_Size_Object;

// A text label that tells me what file I most recently loaded
cLabelPanel *ROI_Position_Checking,*ROI_Insert_Label,*ROI_Delete_Label,*ROI_Increase_Size_Label,*ROI_Decrease_Size_Label;
cLabelPanel *Plane_XY_Label,*Plane_XZ_Label,*Plane_YZ_Label;

// rotational velocity of the torus
cVector3d rotVelocity;

// a 3D cursor which represents the haptic device
cMeta3dofPointer* cursor;

// precision clock to sync dynamic simulation
cPrecisionClock g_clock;

// haptic timer callback
cPrecisionTimer timer;

cProxyPointForceAlgo *proxy;

// width and height of the current viewport display
int width   = 0;
int height  = 0;
int Number_Of_ROIs,Window_Width,Window_Height,MessageNumber;

// menu options
const int OPTION_FULLSCREEN=1;
const int OPTION_WINDOWDISPLAY=2;
const int OPTION_INSERT_NEW_ROI=3;
const int RESTART_HAPTIC=4;
const int MESSAGES=5;

bool Update_ROI_Position=false,Is_Last_New_ROI_Placed_Somewhere,Place_A_ROI;
char *String;
cProxyPointForceAlgo* cur_proxy;
cMatrix3d Invertr_CoorSystem_Rotation,Final_Inserted_ROI_Rotation,Middle_Inserted_ROI_Rotation;
cVector3d Final_Inserted_ROI_Position,Middle_Inserted_ROI_Position;
cMesh *ROI,*Inserted_ROI,*Get_ROI,*Catched_ROI_For_Updation,*Haptic_Tool_object;;
char BUFFER[10];
int Sub_Object_Number;
cVector3d XY_Plane,XZ_Plane,YZ_Plane,Slice_Motion_Checking,Initial_ROI_Rotation;
cVector3d Initial_Slice_Location,Slices_Position;
cGenericObject *Move_SliceObject_Proxy_Contact,*Mirror_Of_Move_SliceObject,*Removed_Object;
double factor,Limit,Initial_Position_Of_XY_Plane;
double Initial_Position_Of_XZ_Plane,Initial_Position_Of_YZ_Plane;
cMatrix3d Invert_Rotation_CoorSystem;
cTexture2D *TorusObjectTexture,*ShapeSphere_Delete_Object_Texture,*ShapeSphere_Add_Object_Texture;
cTexture2D *Increase_ROI_Size_Texture,*Decrease_ROI_Size_Texture;
int FlagStop,Milliseconds_Rate_Update;
cMatrix3d VtK_Rotation_Global,VtK_Rotation_Last,TempMatrix,Change_Coordinates;
double ROI_Scale_Factor,CoorSystem_Rotation_Factor,VtkScaleFactor;
//cTexture2D *XY_Texture,*YZ_Texture,*XZ_Texture;

void hapticsLoop(void* a_pUserData);
//---------------------------------------------------------------------------

void draw(void)
{
    // set the background color of the world
    cColorf color = camera->getParentWorld()->getBackgroundColor();
    glClearColor(color.getR(), color.getG(), color.getB(), color.getA());

    // clear the color and depth buffers
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    // render world
    camera->renderView(width, height);

    // check for any OpenGL errors
    GLenum err;
    err = glGetError();
    if (err != GL_NO_ERROR) printf("Error:  %s\n", gluErrorString(err));

    // Swap buffers
    glutSwapBuffers();
}
//---------------------------------------------------------------------------

int LoadROI()
{
	ROI=new cMesh(world);
	// load 3d object file
	if(ROI->loadFromFile("./3DModels/sphere.3ds")==0){
		printf("Cannot Load The File: %s\n","/3DModels/sphere.3ds");
		return 0;
	}

	strcat(strcpy(String,"ROI_"),_itoa(Number_Of_ROIs,BUFFER,10));
	ROI->setName(String,1);
    // Tell him to compute a bounding box...
	ROI->computeBoundaryBox(true);
	// Build a nice collision-detector for this object
	ROI->createSphereTreeCollisionDetector(true,true);
	// Automatically compute normals for all triangles
	ROI->computeAllNormals(true);
	ROI->computeGlobalPositions(false);
//	ROI->setFrameSize(0.5,1,false);
//	ROI->setShowFrame(true,false);
//	ROI->enableTransparency(0.8, true);
	// Use vertex colors so we can see which triangles collide
	ROI->useColors(true, true);
    ROI->setStiffness(5.0,1);
	ROI->setFriction(0.0,1);
	ROI->m_material.setStaticFriction(0.0);
	CoorSystem->addChild(ROI);
	CoorSystem->computeGlobalPositions(true);
	// printf("Name of the %dst ROI: %s\n",Number_Of_ROIs+1,String);
	Number_Of_ROIs++;
	return 1;
}

//---------------------------------------------------------------------------

void Keyboard(unsigned char key, int x, int y)
{
    if (key == 27){
        // stop the simulation timer
        timer.stop();
        // stop the tool
        cursor->stop();
		free(String);
        // exit application
        exit(0);
    }
	
	if(key =='c') CoorSystem->setHapticEnabled(false,true);
	if(key =='v') CoorSystem->setHapticEnabled(true,true);
	if(key =='w') Milliseconds_Rate_Update+=50;
	if(key =='q')
		if(Milliseconds_Rate_Update>50) Milliseconds_Rate_Update-=50;

	
}
//---------------------------------------------------------------------------

void rezizeWindow(int w, int h)
{
    // update the size of the viewport
    width = w;
    height = h;
    glViewport(0, 0, width, height);
}

//---------------------------------------------------------------------------

void updateDisplay(int val)
{
    // draw scene
    draw();

    // update the GLUT timer for the next rendering call
    glutTimerFunc(50, updateDisplay, 0);
}
//---------------------------------------------------------------------------
void setOther(int value)
{
    switch (value)
    {

	case OPTION_FULLSCREEN:{

            glutFullScreen();
            break;
						   }

	case OPTION_WINDOWDISPLAY:{
            glutReshapeWindow(512, 512);
            glutInitWindowPosition(0, 0);
            break;
							  }

	case OPTION_INSERT_NEW_ROI:{

			// Stop the haptic device...
			timer.stop();
			cursor->setForcesOFF();
			cursor->stop();

			LoadROI();

			// set up the device
			cursor->initialize();
			// open communication to the device
			cursor->start();
			// tell the tool to show his coordinate frame so you
			// can see tool rotation
			cursor->visualizeFrames(true);
			// I need to call this so the tool can update its internal
			// transformations before performing collision detection, etc.
			cursor->computeGlobalPositions();
			cursor->setForcesON();

			world->computeGlobalPositions(true);
			timer.set(0, hapticsLoop, NULL);

            break;
							   }

	case RESTART_HAPTIC:{

			// Stop the haptic device...
			timer.stop();
			cursor->setForcesOFF();
			cursor->stop();

			// set up the device
			cursor->initialize();
			// open communication to the device
			cursor->start();
			// tell the tool to show his coordinate frame so you
			// can see tool rotation
			cursor->visualizeFrames(true);
			// I need to call this so the tool can update its internal
			// transformations before performing collision detection, etc.
			cursor->computeGlobalPositions();
			cursor->setForcesON();

			world->computeGlobalPositions(true);
			timer.set(0, hapticsLoop, NULL);
            break;
						}

	case MESSAGES:{
		
//		printf("CoorSystem Clild Number: %d\n",CoorSystem->getNumChildren());
//		for (Sub_Object_Number = 0; Sub_Object_Number<CoorSystem->getNumChildren(); Sub_Object_Number++){
//			
//			cGenericObject *SubObject=CoorSystem->getChild(Sub_Object_Number);
//			
//			
//			printf("CoorSystem Clild: %d, Name: %s\n",Sub_Object_Number,SubObject->m_objectName);
//			printf("Arxiki Thesi: %f, %f, %f\n\n",SubObject->getPos().x,SubObject->getPos().y,SubObject->getPos().z);
//		}

		
			// printf("cur_proxy->getDeviceGlobalPosition(): (%f,%f,%f)\n",cur_proxy->getDeviceGlobalPosition().x,cur_proxy->getDeviceGlobalPosition().y,cur_proxy->getDeviceGlobalPosition().z);
			// printf("cur_proxy->getProxyGlobalPosition(): (%f,%f,%f)\n\n",cur_proxy->getProxyGlobalPosition().x,cur_proxy->getProxyGlobalPosition().y,cur_proxy->getProxyGlobalPosition().z);
			// printf("cursor->getGlobalPos(): (%f,%f,%f)\n",cursor->getGlobalPos().x,cursor->getGlobalPos().y,cursor->getGlobalPos().z);
			// printf("cursor->getPos(): (%f,%f,%f)\n\n",cursor->getPos().x,cursor->getPos().y,cursor->getPos().z);

		break;
				  }

    }
    
    glutPostRedisplay();
}
//---------------------------------------------------------------------------
void hapticsLoop(void* a_pUserData)
{
	
	ROI_Position_Checking->clearLabels();
	ROI_Position_Checking->addLabel("Running...");

	// Enable the "dynamic proxy", which will handle moving objects
	cur_proxy=dynamic_cast<cProxyPointForceAlgo*>(cursor->getProxy());
	cur_proxy->enableDynamicProxy(true);
	
	Haptic_Tool_object->setPos(cur_proxy->getProxyGlobalPosition());
	Haptic_Tool_object->setRot(cursor->m_deviceGlobalRot);
	HapticMessenger->Emit_UpdateHapticCursor_Signal( cur_proxy->getProxyGlobalPosition() );

	// read position from haptic device
    cursor->updatePose();


	if(cur_proxy!=NULL && cur_proxy->getContactObject()!=NULL){
		if(strstr(cur_proxy->getContactObject()->m_objectName,"_Plane")!=NULL && (!(GetKeyState(VK_SPACE) & (1<<15)))){
			g_clock.stop();
			double increment= g_clock.getCurrentTime() / 1000000.0;
			g_clock.initialize();
			g_clock.start();
			cVector3d cursorPos = cursor->m_deviceGlobalPos;
			cVector3d objectPos = CoorSystem->getGlobalPos();
			cVector3d cursorForce = cursor->m_lastComputedGlobalForce;
			rotVelocity.add( cMul(-10.0 * increment, cCross(cSub(cursorPos, CoorSystem->getGlobalPos()), cursorForce)));
			rotVelocity.mul(1.0 - increment);

			if (rotVelocity.length()>CHAI_SMALL)
			{
				ROI_Position_Checking->clearLabels();
				ROI_Position_Checking->addLabel("The whole object is rotating");

				TorusObject->rotate(cNormalize(rotVelocity), 0.5*CHAI_PI/180.0); //0.3
				CoorSystem->setRot(TorusObject->getRot());

				// send the signal

				CoorSystem->computeGlobalPositions(true);
				//
				//					Change_Coordinates.setCol0(cVector3d(0,1,0));
				//					Change_Coordinates.setCol1(cVector3d(0,0,1));
				//					Change_Coordinates.setCol2(cVector3d(1,0,0));
				//
				//	//				Change_Coordinates.mul(VtK_Rotation_Last);
				//
				//
				//					VtK_Rotation_Last.identity();
				//					VtK_Rotation_Last.rotate(cNormalize(rotVelocity), 0.3*CHAI_PI/180.0);
				//					VtK_Rotation_Global.mul(VtK_Rotation_Last);
				////					TempMatrix.copyfrom(VtK_Rotation_Global);
				////					TempMatrix.mul(Change_Coordinates);
				//					
				//					TempMatrix.copyfrom(Change_Coordinates);
				//					TempMatrix.mul(VtK_Rotation_Global);
				//
				//					HapticMessenger->Emit_SceneRotated_Signal(TempMatrix);
				HapticMessenger->Emit_SceneRotated2_Signal(cNormalize(rotVelocity),0.5);


			}
		}
	}

	
	if(Update_ROI_Position==true && (cursor->m_button & 1)){
		if(Catched_ROI_For_Updation->m_tag==1){

			Catched_ROI_For_Updation->setPos(cur_proxy->getProxyGlobalPosition());
			Catched_ROI_For_Updation->computeGlobalPositions(true);
			Catched_ROI_For_Updation->setRot(cursor->m_deviceGlobalRot);

		

			// For ROI trnasformations
			//Catched_ROI_For_Updation->m_objectName
			//Catched_ROI_For_Updation->getPos()
			//Catched_ROI_For_Updation->getRot().m

			Invertr_CoorSystem_Rotation.identity();
			Invertr_CoorSystem_Rotation.copyfrom(CoorSystem->getGlobalRot());
			Invertr_CoorSystem_Rotation.invert();
			
			Final_Inserted_ROI_Position.copyfrom(Catched_ROI_For_Updation->getPos());
			Middle_Inserted_ROI_Position=Invertr_CoorSystem_Rotation*Final_Inserted_ROI_Position;
			
			Final_Inserted_ROI_Rotation.copyfrom(Catched_ROI_For_Updation->getRot());
			Middle_Inserted_ROI_Rotation=Invertr_CoorSystem_Rotation*Final_Inserted_ROI_Rotation;
			
			ROI_Position_Checking->clearLabels();
			if(abs(Middle_Inserted_ROI_Position.x)>=1 || abs(Middle_Inserted_ROI_Position.y)>=1 || abs(Middle_Inserted_ROI_Position.z)>=1)
				ROI_Position_Checking->addLabel("ROI is out of the volume");
			else {
				strcpy(String,"The position of the ");
				strcat(String,Catched_ROI_For_Updation->m_objectName);
				strcat(String," is modifying");
				ROI_Position_Checking->clearLabels();
				ROI_Position_Checking->addLabel(String);
			}

			TempMatrix.identity();
//			printf("!!!!!!!!!!!!!Transform ROI Me onoma: %s Position: (%f,%f,%f)\n",Catched_ROI_For_Updation->m_objectName,cur_proxy->getProxyGlobalPosition().x,cur_proxy->getProxyGlobalPosition().y,cur_proxy->getProxyGlobalPosition().z);
			HapticMessenger->Emit_TransfromROI_Signal(Catched_ROI_For_Updation->m_objectName,cur_proxy->getProxyGlobalPosition(), TempMatrix);
		}
	}
	
	
	if(cursor->m_button & 1){
		if(Update_ROI_Position==false){
			if(cur_proxy!=NULL && cur_proxy->getContactObject()!=NULL){

				strcpy(String,"Contact with ");
				strcat(String,cur_proxy->getContactObject()->m_objectName);
				strcat(String," object");
				ROI_Position_Checking->clearLabels();
				ROI_Position_Checking->addLabel(String);
				
				if(strstr(cur_proxy->getContactObject()->m_objectName,"ROI_")!=NULL){

					// printf("Epilogi ROI Me onoma: %s sto (%f,%f,%f)\n",cur_proxy->getContactObject()->m_objectName,cur_proxy->getContactObject()->getPos().x,cur_proxy->getContactObject()->getPos().y,cur_proxy->getContactObject()->getPos().z);
					CoorSystem->setHapticEnabled(0,1);
					Catched_ROI_For_Updation=new cMesh(world);
					Catched_ROI_For_Updation->loadFromFile("./3DModels/sphere.3ds");
					Catched_ROI_For_Updation->setName(cur_proxy->getContactObject()->m_objectName,1);
					// Tell him to compute a bounding box...
					Catched_ROI_For_Updation->computeBoundaryBox(true);
					// Build a nice collision-detector for this object
					Catched_ROI_For_Updation->createSphereTreeCollisionDetector(true,true);
					// Automatically compute normals for all triangles
					Catched_ROI_For_Updation->computeAllNormals(true);
					Catched_ROI_For_Updation->computeGlobalPositions(false);
					Catched_ROI_For_Updation->setHapticEnabled(0,1);
//					Catched_ROI_For_Updation->setFrameSize(0.5,1,false);
//					Catched_ROI_For_Updation->setShowFrame(true,false);
				//	Catched_ROI_For_Updation->enableTransparency(0.8, true);
					// Use vertex colors so we can see which triangles collide
					Catched_ROI_For_Updation->useColors(true, true);
					Catched_ROI_For_Updation->setStiffness(5.0,1);
					Catched_ROI_For_Updation->setFriction(0.0,1);
					Catched_ROI_For_Updation->m_material.setStaticFriction(0.0);
					Catched_ROI_For_Updation->m_tag=1;
					world->addChild(Catched_ROI_For_Updation);
					world->computeGlobalPositions();

					for (Sub_Object_Number = 0; Sub_Object_Number<CoorSystem->getNumChildren(); Sub_Object_Number++){
						Removed_Object=CoorSystem->getChild(Sub_Object_Number);
						if(strcmp(Removed_Object->m_objectName,cur_proxy->getContactObject()->m_objectName)==0)
						{ Removed_Object->removeFromGraph(); delete Removed_Object; break;}
					}

					CoorSystem->computeGlobalPositions(true);
					cursor->setForcesOFF();
					Update_ROI_Position=true;
				}
			}
		}

		if(cur_proxy!=NULL && cur_proxy->getContactObject()==NULL && cursor->m_lastComputedGlobalForce.equals(cVector3d(0,0,0))==false){
			if(cur_proxy->getDeviceGlobalPosition().z<0.5 && cur_proxy->getDeviceGlobalPosition().z>-0.5/* && Is_Last_New_ROI_Placed_Somewhere==true*/){
				g_clock.stop();
				double increment= g_clock.getCurrentTime() / 1000000.0;
				g_clock.initialize();
				g_clock.start();
				cVector3d cursorPos = cursor->m_deviceGlobalPos;
				cVector3d objectPos = TorusObject->getGlobalPos();
				cVector3d cursorForce = cursor->m_lastComputedGlobalForce;
				rotVelocity.add( cMul(-10.0 * increment, cCross(cSub(cursorPos, objectPos), cursorForce)));
				rotVelocity.mul(1.0 - increment);
					
				if (rotVelocity.length()>CHAI_SMALL)
				{
					ROI_Position_Checking->clearLabels();
					ROI_Position_Checking->addLabel("The whole object is rotating");

					TorusObject->rotate(cNormalize(rotVelocity), 0.5*CHAI_PI/180.0); //0.3
					CoorSystem->setRot(TorusObject->getRot());

					// send the signal

					CoorSystem->computeGlobalPositions(true);
					//
					//					Change_Coordinates.setCol0(cVector3d(0,1,0));
					//					Change_Coordinates.setCol1(cVector3d(0,0,1));
					//					Change_Coordinates.setCol2(cVector3d(1,0,0));
					//
					//	//				Change_Coordinates.mul(VtK_Rotation_Last);
					//
					//
					//					VtK_Rotation_Last.identity();
					//					VtK_Rotation_Last.rotate(cNormalize(rotVelocity), 0.3*CHAI_PI/180.0);
					//					VtK_Rotation_Global.mul(VtK_Rotation_Last);
					////					TempMatrix.copyfrom(VtK_Rotation_Global);
					////					TempMatrix.mul(Change_Coordinates);
					//					
					//					TempMatrix.copyfrom(Change_Coordinates);
					//					TempMatrix.mul(VtK_Rotation_Global);
					//
					//					HapticMessenger->Emit_SceneRotated_Signal(TempMatrix);
					HapticMessenger->Emit_SceneRotated2_Signal(cNormalize(rotVelocity),0.5);


				}
			}
				
			if(cur_proxy->getDeviceGlobalPosition().y>0.0){
				if(cur_proxy->getDeviceGlobalPosition().z>0.5){
					if(Update_ROI_Position==true){

						// printf("Delete Sphere\n\n");

						Update_ROI_Position=false;

						for (Sub_Object_Number = 0; Sub_Object_Number<world->getNumChildren(); Sub_Object_Number++){			
							Removed_Object=world->getChild(Sub_Object_Number);
							if(strcmp(Removed_Object->m_objectName,Catched_ROI_For_Updation->m_objectName)==0)
							{ Removed_Object->removeFromGraph(); delete Removed_Object; break;}
						}

						Is_Last_New_ROI_Placed_Somewhere=true;
						CoorSystem->setHapticEnabled(1,1);
						cursor->setForcesON();

//						printf("Delete Sphere: %s\n\n",Removed_Object->m_objectName);
						if( HapticMessenger != NULL )
							HapticMessenger->Emit_DeleteROI_Signal(Removed_Object->m_objectName);
					}
				}

				if(cur_proxy->getDeviceGlobalPosition().z<-0.5 && Is_Last_New_ROI_Placed_Somewhere==true){
					// printf("Add Sphere\n\n");
					Is_Last_New_ROI_Placed_Somewhere=false;
					// Stop the haptic device...
 					//timer.stop();
 					//cursor->setForcesOFF();
 					//cursor->stop();
 
					LoadROI();

					//// set up the device
 				//	cursor->initialize();
 				//	// open communication to the device
 				//	cursor->start();
 				//	// tell the tool to show his coordinate frame so you
 				//	// can see tool rotation
 				//	cursor->visualizeFrames(true);
 				//	// I need to call this so the tool can update its internal
 				//	// transformations before performing collision detection, etc.
 				//	cursor->computeGlobalPositions();
 				//	cursor->setForcesON();
 
 				//	timer.set(0, hapticsLoop, NULL);

					// For ADD ROI
					//Catched_ROI_For_Updation->m_objectName
					//Catched_ROI_For_Updation->getPos()
					//Catched_ROI_For_Updation->getRot().m

					// printf("ROI Name: %s Position (%f,%f,%f)  Radius: %f\n",ROI->m_objectName,ROI->getPos().x,ROI->getPos().y,ROI->getPos().z,ROI->getBoundaryMax().x-ROI->getBoundaryCenter().x);

					HapticMessenger->Emit_AddROI_Signal(
						ROI->m_objectName,
						ROI->getPos(),
						ROI->getBoundaryMax().x-ROI->getBoundaryCenter().x );

				}
			}
			else{

				if(cur_proxy->getDeviceGlobalPosition().z>0.5 && Update_ROI_Position==true){
					Catched_ROI_For_Updation->scale(cVector3d(1+ROI_Scale_Factor,1+ROI_Scale_Factor,1+ROI_Scale_Factor)); //1.00005 
					cursor->setForcesON();

					Catched_ROI_For_Updation->computeGlobalPositions(true);
					// printf("1. Allazo sxima sto ROI %f\n\n");

					VtkScaleFactor*=1+ROI_Scale_Factor;

					HapticMessenger->Emit_ScaleROI_Signal(Catched_ROI_For_Updation->m_objectName,VtkScaleFactor);

				}

				if(cur_proxy->getDeviceGlobalPosition().z<0.5 && Update_ROI_Position==true){
					Catched_ROI_For_Updation->scale(cVector3d(1-ROI_Scale_Factor,1-ROI_Scale_Factor,1-ROI_Scale_Factor)); //0.99995 
					cursor->setForcesON();
					Catched_ROI_For_Updation->computeGlobalPositions(true);
					// printf("2. Allazo sxima sto ROI\n\n");
					VtkScaleFactor*=1-ROI_Scale_Factor;
					HapticMessenger->Emit_ScaleROI_Signal(Catched_ROI_For_Updation->m_objectName,VtkScaleFactor);
				}
			}
		}

	}else{
		if(Update_ROI_Position==true){
			Update_ROI_Position=false;
			
			Invertr_CoorSystem_Rotation.identity();
			Invertr_CoorSystem_Rotation.copyfrom(CoorSystem->getGlobalRot());
			Invertr_CoorSystem_Rotation.invert();

			Final_Inserted_ROI_Position.copyfrom(Catched_ROI_For_Updation->getPos());
			Middle_Inserted_ROI_Position=Invertr_CoorSystem_Rotation*Final_Inserted_ROI_Position;

			Final_Inserted_ROI_Rotation.copyfrom(Catched_ROI_For_Updation->getRot());
			Middle_Inserted_ROI_Rotation=Invertr_CoorSystem_Rotation*Final_Inserted_ROI_Rotation;

			ROI_Position_Checking->clearLabels();
			if(abs(Middle_Inserted_ROI_Position.x)>=1 || abs(Middle_Inserted_ROI_Position.y)>=1 || abs(Middle_Inserted_ROI_Position.z)>=1){
				 ROI_Position_Checking->addLabel("ROI is out of the volume");
				 Catched_ROI_For_Updation->setPos(cVector3d(0,0,0));
				 ROI_Position_Checking->addLabel("Running...");
			}
			else ROI_Position_Checking->addLabel("Running...");

			// printf("Catched_ROI_For_Updation->getPos(): (%f,%f,%f)\n",Catched_ROI_For_Updation->getPos().x,Catched_ROI_For_Updation->getPos().y,Catched_ROI_For_Updation->getPos().z);
			// printf("Catched_ROI_For_Updation->getGlobalPos(): (%f,%f,%f)\n\n",Catched_ROI_For_Updation->getGlobalPos().x,Catched_ROI_For_Updation->getGlobalPos().y,Catched_ROI_For_Updation->getGlobalPos().z);
			// printf("cur_proxy->getDeviceGlobalPosition(): (%f,%f,%f)\n",cur_proxy->getDeviceGlobalPosition().x,cur_proxy->getDeviceGlobalPosition().y,cur_proxy->getDeviceGlobalPosition().z);
			// printf("cur_proxy->getProxyGlobalPosition(): (%f,%f,%f)\n\n",cur_proxy->getProxyGlobalPosition().x,cur_proxy->getProxyGlobalPosition().y,cur_proxy->getProxyGlobalPosition().z);
			// printf("cursor->getGlobalPos(): (%f,%f,%f)\n",cursor->getGlobalPos().x,cursor->getGlobalPos().y,cursor->getGlobalPos().z);
			// printf("cursor->getPos(): (%f,%f,%f)\n\n",cursor->getPos().x,cursor->getPos().y,cursor->getPos().z);

			Invertr_CoorSystem_Rotation.identity();
			Invertr_CoorSystem_Rotation.copyfrom(CoorSystem->getGlobalRot());
			Invertr_CoorSystem_Rotation.invert();

			Final_Inserted_ROI_Position.copyfrom(Catched_ROI_For_Updation->getPos());
			Middle_Inserted_ROI_Position=Invertr_CoorSystem_Rotation*Final_Inserted_ROI_Position;

			Final_Inserted_ROI_Rotation.copyfrom(Catched_ROI_For_Updation->getRot());
			Middle_Inserted_ROI_Rotation=Invertr_CoorSystem_Rotation*Final_Inserted_ROI_Rotation;

			Catched_ROI_For_Updation->m_tag=0;
			Inserted_ROI=new cMesh(world);
			Inserted_ROI->loadFromFile("./3DModels/sphere.3ds");
			Inserted_ROI->setName(Catched_ROI_For_Updation->m_objectName,1);

			// Tell him to compute a bounding box...
			Inserted_ROI->computeBoundaryBox(true);
			// Build a nice collision-detector for this object
			Inserted_ROI->createSphereTreeCollisionDetector(true,true);
			// Automatically compute normals for all triangles
			Inserted_ROI->computeAllNormals(true);


			Inserted_ROI->scale(cVector3d(Catched_ROI_For_Updation->getBoundaryMin().x/Inserted_ROI->getBoundaryMin().x,
											Catched_ROI_For_Updation->getBoundaryMin().y/Inserted_ROI->getBoundaryMin().y,
											Catched_ROI_For_Updation->getBoundaryMin().z/Inserted_ROI->getBoundaryMin().z));

			Inserted_ROI->computeGlobalPositions(false);
//			Inserted_ROI->setFrameSize(0.5,1,false);
//			Inserted_ROI->setShowFrame(true,false);
			//	Inserted_ROI->enableTransparency(0.8, true);
			// Use vertex colors so we can see which triangles collide
			Inserted_ROI->useColors(true, true);
			Inserted_ROI->setStiffness(5.0,1);
			Inserted_ROI->setFriction(0.0,1);
			Inserted_ROI->m_material.setStaticFriction(0.0);
			CoorSystem->addChild(Inserted_ROI);
			CoorSystem->computeGlobalPositions(true);

			// printf("ROI: (%f,%f,%f)\n",Inserted_ROI->getPos().x,Inserted_ROI->getPos().y,Inserted_ROI->getPos().z);
			CoorSystem->getChild(CoorSystem->getNumChildren()-1)->setRot(Middle_Inserted_ROI_Rotation);
//			CoorSystem->getChild(CoorSystem->getNumChildren()-1)->setPos(cMul(cTrans(CoorSystem->getGlobalRot()), cSub(cur_proxy->getDeviceGlobalPosition(), CoorSystem->getGlobalPos())));
			CoorSystem->getChild(CoorSystem->getNumChildren()-1)->setPos(Middle_Inserted_ROI_Position);
			CoorSystem->computeGlobalPositions(true);
			// printf("NeW_ROI: %s sto (%f,%f,%f)\n",CoorSystem->getChild(CoorSystem->getNumChildren()-1)->m_objectName,Catched_ROI_For_Updation->getGlobalPos().x,Catched_ROI_For_Updation->getGlobalPos().y,Catched_ROI_For_Updation->getGlobalPos().z);

			for (Sub_Object_Number = 0; Sub_Object_Number<world->getNumChildren(); Sub_Object_Number++){
				
				Removed_Object=world->getChild(Sub_Object_Number);				
				if(strcmp(Removed_Object->m_objectName,world->getChild(world->getNumChildren()-1)->m_objectName)==0)
				{ Removed_Object->removeFromGraph(); delete Removed_Object; break;}
			}
			Is_Last_New_ROI_Placed_Somewhere=true;
			CoorSystem->setHapticEnabled(1,1);
			cursor->setForcesON();
		}
		else
			if(cur_proxy!=NULL && cur_proxy->getContactObject()!=NULL && (GetKeyState(VK_SPACE) & (1<<15))){

				Move_SliceObject_Proxy_Contact=cur_proxy->getContactObject();
			
				if(strstr(Move_SliceObject_Proxy_Contact->m_objectName,"_Plane")!=NULL){	

					if(strstr(Move_SliceObject_Proxy_Contact->m_objectName,"XY_Plane")!=NULL){
						
						for (Sub_Object_Number = 0; Sub_Object_Number<CoorSystem->getNumChildren(); Sub_Object_Number++){
							cGenericObject *SubObject=CoorSystem->getChild(Sub_Object_Number);
							if(strcmp(Move_SliceObject_Proxy_Contact->m_objectName,SubObject->m_objectName)!=0){
								SubObject->setHapticEnabled(0,1);
								if(strstr(SubObject->m_objectName,"XY_Plane")!=NULL)
									Mirror_Of_Move_SliceObject=CoorSystem->getChild(Sub_Object_Number);
							}
						}
						
						//___________________________________Motion Along Y axis____________________________
						// printf("Move XY_Plane\n");
						// printf("Arxiki Thesi: %s %f, %f, %f\n",Move_SliceObject_Proxy_Contact->m_objectName,Move_SliceObject_Proxy_Contact->getPos().x,Move_SliceObject_Proxy_Contact->getPos().y,Move_SliceObject_Proxy_Contact->getPos().z);
						// printf("Arxiki Thesi katoptrou: %s %f, %f, %f\n\n",Mirror_Of_Move_SliceObject->m_objectName,Mirror_Of_Move_SliceObject->getPos().x,Mirror_Of_Move_SliceObject->getPos().y,Mirror_Of_Move_SliceObject->getPos().z);

						ROI_Position_Checking->clearLabels();
						ROI_Position_Checking->addLabel("Changing position of the XY Plane");

						XY_Plane.set(0,0,1);
						if(strcmp("XY_Plane_1",Move_SliceObject_Proxy_Contact->m_objectName)==0) {factor=-0.01; Limit=0.001;}
						else {factor=0.01; Limit=-0.001;}
						XY_Plane.elementMul(cVector3d(0,0,factor));
//						XY_Plane=CoorSystem->getGlobalRot()*XY_Plane;
						Slice_Motion_Checking.copyfrom(Move_SliceObject_Proxy_Contact->getPos());
						
						// printf("Initial Slice_Motion_Checking.z: %f\n\n",Slice_Motion_Checking.z);
						Slice_Motion_Checking.add(XY_Plane);	
						// printf("Added Value: %f, %f, %f\n",XY_Plane.x,XY_Plane.y,XY_Plane.z);
						
						// printf("Slice_Motion_Checking.y: %f\n\n",Slice_Motion_Checking.z);
						
						if(Slice_Motion_Checking.z<=1+Limit && Slice_Motion_Checking.z>=-1+Limit){
							Mirror_Of_Move_SliceObject->translate(XY_Plane);
							Mirror_Of_Move_SliceObject->computeGlobalPositions(true);
							Move_SliceObject_Proxy_Contact->translate(XY_Plane);
							Move_SliceObject_Proxy_Contact->computeGlobalPositions(true);
						}

						Slices_Position.z=Move_SliceObject_Proxy_Contact->getPos().z;

						MessageNumber++;
						// printf("Message Number: %d\n",MessageNumber);
						// send the message to the VTK
						if( HapticMessenger != NULL )
							HapticMessenger->Emit_ChangeSlicePosition_Signal( Slices_Position );


						// printf("Teliki Thesi: %s %f, %f, %f\n",Move_SliceObject_Proxy_Contact->m_objectName,Move_SliceObject_Proxy_Contact->getPos().x,Move_SliceObject_Proxy_Contact->getPos().y,Move_SliceObject_Proxy_Contact->getPos().z);
						// printf("Teliki Thesi Tou Katoptrou: %s %f, %f, %f\n",Mirror_Of_Move_SliceObject->m_objectName,Mirror_Of_Move_SliceObject->getPos().x,Mirror_Of_Move_SliceObject->getPos().y,Mirror_Of_Move_SliceObject->getPos().z);

					}
					else if(strstr(Move_SliceObject_Proxy_Contact->m_objectName,"XZ_Plane")!=NULL){
						
						// printf("Onoma epilogis: %s\n",Move_SliceObject_Proxy_Contact->m_objectName);
						for (Sub_Object_Number = 0; Sub_Object_Number<CoorSystem->getNumChildren(); Sub_Object_Number++){
							
							cGenericObject *SubObject=CoorSystem->getChild(Sub_Object_Number);
							if(strcmp(Move_SliceObject_Proxy_Contact->m_objectName,SubObject->m_objectName)!=0){
								
								SubObject->setHapticEnabled(0,1);
								if(strstr(SubObject->m_objectName,"XZ_Plane")!=NULL){
									Mirror_Of_Move_SliceObject=CoorSystem->getChild(Sub_Object_Number);
									// printf("Onoma katoptrou: %s\n",Mirror_Of_Move_SliceObject->m_objectName);
								}
							}
						}
						
						// printf("Move XZ_Plane\n");
						// printf("Arxiki Thesi: %s %f, %f, %f\n",Move_SliceObject_Proxy_Contact->m_objectName,Move_SliceObject_Proxy_Contact->getPos().x,Move_SliceObject_Proxy_Contact->getPos().y,Move_SliceObject_Proxy_Contact->getPos().z);
						// printf("Arxiki Thesi katoptrou: %s %f, %f, %f\n\n",Mirror_Of_Move_SliceObject->m_objectName,Mirror_Of_Move_SliceObject->getPos().x,Mirror_Of_Move_SliceObject->getPos().y,Mirror_Of_Move_SliceObject->getPos().z);

						ROI_Position_Checking->clearLabels();
						ROI_Position_Checking->addLabel("Changing position of the XZ Plane");

						XZ_Plane.set(0,1,0);
						if(strcmp("XZ_Plane_1",Move_SliceObject_Proxy_Contact->m_objectName)==0) {factor=-0.01; Limit=-0.001;}
						else {factor=0.01; Limit=0.001;}
						XZ_Plane.elementMul(cVector3d(0,factor,0));
//						XZ_Plane=CoorSystem->getGlobalRot()*XZ_Plane;
						Slice_Motion_Checking.copyfrom(Move_SliceObject_Proxy_Contact->getPos());

						// printf("Initial Slice_Motion_Checking.y: %f\n\n",Slice_Motion_Checking.y);
						Slice_Motion_Checking.add(XZ_Plane);						
						// printf("Added Value: %f, %f, %f\n",XZ_Plane.x,XZ_Plane.y,XZ_Plane.z);
						
						// printf("Slice_Motion_Checking.z: %f\n\n",Slice_Motion_Checking.y);

						if(Slice_Motion_Checking.y<=1+Limit && Slice_Motion_Checking.y>=-1+Limit){
							Mirror_Of_Move_SliceObject->translate(XZ_Plane);
							Mirror_Of_Move_SliceObject->computeGlobalPositions(true);
							Move_SliceObject_Proxy_Contact->translate(XZ_Plane);
							Move_SliceObject_Proxy_Contact->computeGlobalPositions(true);
						}
	
						Slices_Position.y=Move_SliceObject_Proxy_Contact->getPos().y;

						MessageNumber++;
						// printf("Message Number: %d\n",MessageNumber);
						// send the message to the VTK
						if( HapticMessenger != NULL )
							HapticMessenger->Emit_ChangeSlicePosition_Signal( Slices_Position );


						// printf("Teliki Thesi: %s %f, %f, %f\n",Move_SliceObject_Proxy_Contact->m_objectName,Move_SliceObject_Proxy_Contact->getPos().x,Move_SliceObject_Proxy_Contact->getPos().y,Move_SliceObject_Proxy_Contact->getPos().z);
						// printf("Teliki Thesi Tou Katoptrou: %s %f, %f, %f\n",Mirror_Of_Move_SliceObject->m_objectName,Mirror_Of_Move_SliceObject->getPos().x,Mirror_Of_Move_SliceObject->getPos().y,Mirror_Of_Move_SliceObject->getPos().z);
						
					}
					else if(strstr(Move_SliceObject_Proxy_Contact->m_objectName,"YZ_Plane")!=NULL){
						
						for (Sub_Object_Number = 0; Sub_Object_Number<CoorSystem->getNumChildren(); Sub_Object_Number++){
							
							cGenericObject *SubObject=CoorSystem->getChild(Sub_Object_Number);
							if(strcmp(Move_SliceObject_Proxy_Contact->m_objectName,SubObject->m_objectName)!=0){
								
								SubObject->setHapticEnabled(0,1);
								if(strstr(SubObject->m_objectName,"YZ_Plane")!=NULL)
									Mirror_Of_Move_SliceObject=CoorSystem->getChild(Sub_Object_Number);
							}
						}
						
						ROI_Position_Checking->clearLabels();
						ROI_Position_Checking->addLabel("Changing position of the YZ Plane");

						YZ_Plane.set(1,0,0);
						if(strcmp("YZ_Plane_1",Move_SliceObject_Proxy_Contact->m_objectName)==0){ factor=-0.01; Limit=0.001;}
						else{ factor=0.01; Limit=-0.001;}
						YZ_Plane.elementMul(cVector3d(factor,0,0));
//						YZ_Plane=CoorSystem->getGlobalRot()*YZ_Plane;
						Slice_Motion_Checking.copyfrom(Move_SliceObject_Proxy_Contact->getPos());

						// printf("Initial Slice_Motion_Checking.x: %f\n\n",Slice_Motion_Checking.x);
						Slice_Motion_Checking.add(YZ_Plane);						
						// printf("Added Value: %f, %f, %f\n",YZ_Plane.x,YZ_Plane.y,YZ_Plane.z);
						
						// printf("Slice_Motion_Checking.x: %f\n\n",Slice_Motion_Checking.x);
						
						if(Slice_Motion_Checking.x<=1+Limit && Slice_Motion_Checking.x>=-1+Limit){
							Mirror_Of_Move_SliceObject->translate(YZ_Plane);
							Mirror_Of_Move_SliceObject->computeGlobalPositions(true);
							Move_SliceObject_Proxy_Contact->translate(YZ_Plane);
							Move_SliceObject_Proxy_Contact->computeGlobalPositions(true);
						}

						Slices_Position.x=Move_SliceObject_Proxy_Contact->getPos().x;

						MessageNumber++;
						// printf("Message Number: %d\n",MessageNumber);
						// send the message to the VTK
						if( HapticMessenger != NULL )
							HapticMessenger->Emit_ChangeSlicePosition_Signal( Slices_Position );

										
						// printf("Teliki Thesi: %s %f, %f, %f\n",Move_SliceObject_Proxy_Contact->m_objectName,Move_SliceObject_Proxy_Contact->getPos().x,Move_SliceObject_Proxy_Contact->getPos().y,Move_SliceObject_Proxy_Contact->getPos().z);
						// printf("Teliki Thesi Tou Katoptrou: %s %f, %f, %f\n",Mirror_Of_Move_SliceObject->m_objectName,Mirror_Of_Move_SliceObject->getPos().x,Mirror_Of_Move_SliceObject->getPos().y,Mirror_Of_Move_SliceObject->getPos().z);
					}
			}
			
			CoorSystem->computeGlobalPositions(true);
			CoorSystem->setHapticEnabled(1,1);
		}
	}

	
	// compute forces
	cursor->computeForces();
	// send forces to haptic device
	cursor->applyForces();

//	Sleep(Milliseconds_Rate_Update);
}

//---------------------------------------------------------------------------

int main(int argc, char* argv[])
{
    // display pretty message
    printf ("\n");
    printf ("  ============================================================\n");
    printf ("  Multi-Approach DT-MRI Data Analysis & Visualization Platform\n");
    printf ("  ============================================================\n");
    printf ("\n");

	String=(char*)malloc(_MAX_PATH*sizeof(char*));
	Is_Last_New_ROI_Placed_Somewhere=true;
	Number_Of_ROIs=0;
	Milliseconds_Rate_Update=100;
	MessageNumber=0;
	VtK_Rotation_Global.identity();
	Place_A_ROI=false;
	Window_Width=800; Window_Height=800;
	ROI_Scale_Factor=0.0001;
	VtkScaleFactor=1;

	Initial_Position_Of_XY_Plane=0.0;
	Initial_Position_Of_XZ_Plane=0.0;
	Initial_Position_Of_YZ_Plane=0.0;

	Slices_Position.set(0,0,0);

	//======================WORLD======================
    // create a new world
    world = new cWorld();
    // set background color
    world->setBackgroundColor(0.4f,0.4f,0.4f);
    // create a camera
    camera = new cCamera(world);
    world->addChild(camera);

//	XY_Texture = world->newTexture();
//	YZ_Texture = world->newTexture();
//	XZ_Texture = world->newTexture();

    // position a camera
    camera->set(cVector3d (5, 0.0, 0.0),
                cVector3d (0.0, 0.0, 0.0),
                cVector3d (0.0, 0.0, 1.0));

	// set the near and far clipping planes of the camera
	//   camera->setClippingPlanes(0.01, 10.0);

    // Create a light source and attach it to the camera
    light = new cLight(world);
    light->setEnabled(true);
//	light->setDirectionalLight(true);
    light->setPos(cVector3d(2,0.5,1));
	light->setDir(cVector3d(-2,0.5,1));
    camera->addChild(light);

	Plane_XY_Label = new cLabelPanel(world);
	Plane_XY_Label->addLabel("Red Plane   : XY Plane");
	Plane_XY_Label->getFont()->setPointSize(1.0f);
	Plane_XY_Label->getFont()->setFontFace("arial");
	Plane_XY_Label->setBorders(10,10,0,0,0);
	Plane_XY_Label->setDisplayRect(true);
	Plane_XY_Label->m_rectColor = cColorf(1.0,0.0,0.0);
	Plane_XY_Label->setDisplayEdges(true);
	Plane_XY_Label->m_edgeColor = cColorf(0.5,0.5,0.5);
	Plane_XY_Label->m_textColor = cColorf(1,1,1);    
	Plane_XY_Label->setPos(cVector3d(1,Window_Height-25,0));
	Plane_XY_Label->m_useLighting = false;
	Plane_XY_Label->setAlignment(ALIGN_LEFT,VALIGN_BOTTOM);
	camera->m_front_2Dscene.addChild(Plane_XY_Label);

	Plane_XZ_Label = new cLabelPanel(world);
	Plane_XZ_Label->addLabel("Green Plane: XZ Plane");
	Plane_XZ_Label->getFont()->setPointSize(1.0f);
	Plane_XZ_Label->getFont()->setFontFace("arial");
	Plane_XZ_Label->setBorders(10,10,0,0,0);
	Plane_XZ_Label->setDisplayRect(true);
	Plane_XZ_Label->m_rectColor = cColorf(0.0,1.0,0.0);
	Plane_XZ_Label->setDisplayEdges(true);
	Plane_XZ_Label->m_edgeColor = cColorf(0.5,0.5,0.5);
	Plane_XZ_Label->m_textColor = cColorf(1,1,1);    
	Plane_XZ_Label->setPos(cVector3d(1,Window_Height-50,0)); //95
	Plane_XZ_Label->m_useLighting = false;
	Plane_XZ_Label->setAlignment(ALIGN_LEFT,VALIGN_BOTTOM);
	camera->m_front_2Dscene.addChild(Plane_XZ_Label);

	Plane_YZ_Label = new cLabelPanel(world);
	Plane_YZ_Label->addLabel("Blue Plane   : YZ Plane");
	Plane_YZ_Label->getFont()->setPointSize(1.0f);
	Plane_YZ_Label->getFont()->setFontFace("arial");
	Plane_YZ_Label->setBorders(10,10,0,0,0);
	Plane_YZ_Label->setDisplayRect(true);
	Plane_YZ_Label->m_rectColor = cColorf(0.0,0.0,1.0);
	Plane_YZ_Label->setDisplayEdges(true);
	Plane_YZ_Label->m_edgeColor = cColorf(0.5,0.5,0.5);
	Plane_YZ_Label->m_textColor = cColorf(1,1,1);    
	Plane_YZ_Label->setPos(cVector3d(1,Window_Height-75,0)); //200
	Plane_YZ_Label->m_useLighting = false;
	Plane_YZ_Label->setAlignment(ALIGN_LEFT,VALIGN_BOTTOM);
	camera->m_front_2Dscene.addChild(Plane_YZ_Label);

	ROI_Insert_Label = new cLabelPanel(world);
	ROI_Insert_Label->addLabel("Lower Right Sphere: Insert New ROI      ");
	ROI_Insert_Label->getFont()->setPointSize(1.0f);
	ROI_Insert_Label->getFont()->setFontFace("arial");
	ROI_Insert_Label->setBorders(10,10,0,0,0);
	ROI_Insert_Label->setDisplayRect(true);
//	ROI_Insert_Label->m_rectColor = cColorf(0.3,0.3,0.3);
    ROI_Insert_Label->m_material.setShininess(100);
    ROI_Insert_Label->m_material.m_ambient.set((float)0.3, (float)0.3, (float)0.3);
    ROI_Insert_Label->m_material.m_diffuse.set((float)0.1, (float)0.7, (float)0.8);
    ROI_Insert_Label->m_material.m_specular.set((float)1.0, (float)1.0, (float)1.0);
	ROI_Insert_Label->setDisplayEdges(true);
	ROI_Insert_Label->m_edgeColor = cColorf(0.1,0.7,0.8);
	ROI_Insert_Label->m_textColor = cColorf(1,1,1);    
	ROI_Insert_Label->setPos(cVector3d(110,Window_Height-25,0));
	ROI_Insert_Label->m_useLighting = false;
	ROI_Insert_Label->setAlignment(ALIGN_LEFT,VALIGN_BOTTOM);
	camera->m_front_2Dscene.addChild(ROI_Insert_Label);
	

	ROI_Delete_Label = new cLabelPanel(world);
	ROI_Delete_Label->addLabel("Upper Right Sphere: Delete ROI             ");
	ROI_Delete_Label->getFont()->setPointSize(1.0f);
	ROI_Delete_Label->getFont()->setFontFace("arial");
	ROI_Delete_Label->setBorders(10,10,0,0,0);
	ROI_Delete_Label->setDisplayRect(true);
//	ROI_Delete_Label->m_rectColor = cColorf(1.0,0.0,0.0);
    ROI_Insert_Label->m_material.setShininess(100);
    ROI_Insert_Label->m_material.m_ambient.set((float)0.3, (float)0.3, (float)0.3);
    ROI_Insert_Label->m_material.m_diffuse.set((float)0.1, (float)0.7, (float)0.8);
    ROI_Insert_Label->m_material.m_specular.set((float)1.0, (float)1.0, (float)1.0);
	ROI_Delete_Label->setDisplayEdges(true);
	ROI_Delete_Label->m_edgeColor = cColorf(0.1,0.7,0.8);
	ROI_Delete_Label->m_textColor = cColorf(1,1,1);    
	ROI_Delete_Label->setPos(cVector3d(110,Window_Height-50,0));
	ROI_Delete_Label->m_useLighting = false;
	ROI_Delete_Label->setAlignment(ALIGN_LEFT,VALIGN_BOTTOM);
	camera->m_front_2Dscene.addChild(ROI_Delete_Label);


	ROI_Decrease_Size_Label = new cLabelPanel(world);
	ROI_Decrease_Size_Label->addLabel("Lower Left Sphere: Decrease ROI's Size ");
	ROI_Decrease_Size_Label->getFont()->setPointSize(1.0f);
	ROI_Decrease_Size_Label->getFont()->setFontFace("arial");
	ROI_Decrease_Size_Label->setBorders(10,10,0,0,0);
	ROI_Decrease_Size_Label->setDisplayRect(true);
//	ROI_Decrease_Size_Label->m_rectColor = cColorf(1.0,0.0,0.0);
    ROI_Decrease_Size_Label->m_material.setShininess(100);
    ROI_Decrease_Size_Label->m_material.m_ambient.set((float)0.3, (float)0.3, (float)0.3);
    ROI_Decrease_Size_Label->m_material.m_diffuse.set((float)0.1, (float)0.7, (float)0.8);
    ROI_Decrease_Size_Label->m_material.m_specular.set((float)1.0, (float)1.0, (float)1.0);
	ROI_Decrease_Size_Label->setDisplayEdges(true);
	ROI_Decrease_Size_Label->m_edgeColor = cColorf(0.1,0.7,0.8);
	ROI_Decrease_Size_Label->m_textColor = cColorf(1,1,1);    
	ROI_Decrease_Size_Label->setPos(cVector3d(205,Window_Height-65,0));
	ROI_Decrease_Size_Label->m_useLighting = false;
	ROI_Delete_Label->setAlignment(ALIGN_LEFT,VALIGN_BOTTOM);
	camera->m_front_2Dscene.addChild(ROI_Decrease_Size_Label);


	ROI_Increase_Size_Label = new cLabelPanel(world);
	ROI_Increase_Size_Label->addLabel("Lower Right Sphere: Increase ROI's Size");
	ROI_Increase_Size_Label->getFont()->setPointSize(1.0f);
	ROI_Increase_Size_Label->getFont()->setFontFace("arial");
	ROI_Increase_Size_Label->setBorders(10,10,0,0,0);
	ROI_Increase_Size_Label->setDisplayRect(true);
//	ROI_Increase_Size_Label->m_rectColor = cColorf(1.0,0.0,0.0);
    ROI_Increase_Size_Label->m_material.setShininess(100);
    ROI_Increase_Size_Label->m_material.m_ambient.set((float)0.3, (float)0.3, (float)0.3);
    ROI_Increase_Size_Label->m_material.m_diffuse.set((float)0.1, (float)0.7, (float)0.8);
    ROI_Increase_Size_Label->m_material.m_specular.set((float)1.0, (float)1.0, (float)1.0);
	ROI_Increase_Size_Label->setDisplayEdges(true);
	ROI_Increase_Size_Label->m_edgeColor = cColorf(0.1,0.7,0.8);
	ROI_Increase_Size_Label->m_textColor = cColorf(1,1,1);    
	ROI_Increase_Size_Label->setPos(cVector3d(204,Window_Height-90,0));
	ROI_Increase_Size_Label->m_useLighting = false;
	ROI_Delete_Label->setAlignment(ALIGN_LEFT,VALIGN_BOTTOM);
	camera->m_front_2Dscene.addChild(ROI_Increase_Size_Label);


	ROI_Position_Checking = new cLabelPanel(world);
	ROI_Position_Checking->addLabel("Running...");
	ROI_Position_Checking->getFont()->setPointSize(20.0f);
	ROI_Position_Checking->getFont()->setFontFace("arial");
	ROI_Position_Checking->setBorders(15,15,-1,-1,-1);
	ROI_Position_Checking->setDisplayRect(true);
	ROI_Position_Checking->m_rectColor = cColorf(0.7,0.2,0.2);
	ROI_Position_Checking->setDisplayEdges(true);
	ROI_Position_Checking->m_edgeColor = cColorf(0.5,0.5,0.5);
	ROI_Position_Checking->m_textColor = cColorf(1,1,1);    
	ROI_Position_Checking->setPos(cVector3d(310,Window_Height-50,0));
	ROI_Position_Checking->m_useLighting = false;
	ROI_Position_Checking->setAlignment(ALIGN_LEFT,VALIGN_BOTTOM);
	camera->m_front_2Dscene.addChild(ROI_Position_Checking);
	//=================================================

	//======================OBJECTS======================

	Haptic_Tool_object= new cMesh(world);
	Haptic_Tool_object->loadFromFile(".//3DModels//HapticTool.3ds");
	Haptic_Tool_object->setName("Haptic_Tool",false);
	Haptic_Tool_object->computeGlobalPositions(true);
	Haptic_Tool_object->useMaterial(true,true);
	Haptic_Tool_object->setHapticEnabled(0,1);
	Haptic_Tool_object->rotate(cVector3d(1,0,0),CHAI_PI);
	Haptic_Tool_object->scale(cVector3d(0.02,0.02,0.02),true);
	Haptic_Tool_object->finalize(true);
    world->addChild(Haptic_Tool_object);	
	world->computeGlobalPositions();

		
    // load 3d object file
    CoorSystem=new cMesh(world);
    CoorSystem->loadFromFile(".//3DModels//System.obj");
	CoorSystem->setName("CoorSystem",false);
    // Tell him to compute a bounding box...
	CoorSystem->computeBoundaryBox(true);
	// Build a nice collision-detector for this object
	CoorSystem->createSphereTreeCollisionDetector(true,true);
	// Automatically compute normals for all triangles
	CoorSystem->computeAllNormals(true);
	CoorSystem->computeGlobalPositions(true);
//	CoorSystem->setFrameSize(0.5,1,false);
//	CoorSystem->setShowFrame(true,false);
//	CoorSystem->enableTransparency(0.8, true);
	// Use vertex colors so we can see which triangles collide
//	CoorSystem->useColors(true, true);
    CoorSystem->setStiffness(5.0,1);
	CoorSystem->setFriction(0.2,1);
	CoorSystem->m_material.setStaticFriction(0.2);
	CoorSystem->useMaterial(true,true);
    world->addChild(CoorSystem);	
	world->computeGlobalPositions();

	printf("CoorSystem Clild Number: %d\n",CoorSystem->getNumChildren());

	for (Sub_Object_Number = 0; Sub_Object_Number<CoorSystem->getNumChildren(); Sub_Object_Number++){
		

		cGenericObject *SubObject=CoorSystem->getChild(Sub_Object_Number);

		if(Sub_Object_Number==0) SubObject->setName("XZ_Plane_1",true);
		if(Sub_Object_Number==1) SubObject->setName("XZ_Plane_2",true);

 		if(Sub_Object_Number==2) SubObject->setName("XY_Plane_1",true);
 		if(Sub_Object_Number==3) SubObject->setName("XY_Plane_2",true);
 
 		if(Sub_Object_Number==4) SubObject->setName("YZ_Plane_1",true);
 		if(Sub_Object_Number==5) SubObject->setName("YZ_Plane_2",true);
	}


	for (Sub_Object_Number = 0; Sub_Object_Number<CoorSystem->getNumChildren(); Sub_Object_Number++){
		
		cMesh *SubObject=(cMesh *)CoorSystem->getChild(Sub_Object_Number);

		Initial_Slice_Location.copyfrom(cVector3d(0,Initial_Position_Of_XZ_Plane+0.001,0));
		Initial_Slice_Location=CoorSystem->getRot()*Initial_Slice_Location;
		if(strcmp(SubObject->m_objectName,"XZ_Plane_1")==0){
			SubObject->setPos(Initial_Slice_Location);
//			XZ_Texture->loadFromFile("./Texture/COKE.bmp");
//			SubObject->setTexture(XZ_Texture,0);
//			SubObject->useTexture(1,0);
		}

		Initial_Slice_Location.copyfrom(cVector3d(0,Initial_Position_Of_XZ_Plane-0.001,0));
		Initial_Slice_Location=CoorSystem->getRot()*Initial_Slice_Location;
		if(strcmp(SubObject->m_objectName,"XZ_Plane_2")==0){
			SubObject->setPos(Initial_Slice_Location);
//			SubObject->setTexture(XZ_Texture,0);
//			SubObject->useTexture(1,0);
		}
		
		Initial_Slice_Location.copyfrom(cVector3d(0,0,Initial_Position_Of_XY_Plane+0.001));
		Initial_Slice_Location=CoorSystem->getRot()*Initial_Slice_Location;
		if(strcmp(SubObject->m_objectName,"XY_Plane_1")==0){
			SubObject->setPos(Initial_Slice_Location);
//			XY_Texture->loadFromFile("./Texture/COKE.bmp");
//			SubObject->setTexture(XY_Texture,0);
//			SubObject->useTexture(1,0);
		}

		Initial_Slice_Location.copyfrom(cVector3d(0,0,Initial_Position_Of_XY_Plane-0.001));
		Initial_Slice_Location=CoorSystem->getRot()*Initial_Slice_Location;
		if(strcmp(SubObject->m_objectName,"XY_Plane_2")==0){
			SubObject->setPos(Initial_Slice_Location);
//			SubObject->setTexture(XY_Texture,0);
//			SubObject->useTexture(1,0);
		}
		
		Initial_Slice_Location.copyfrom(cVector3d(Initial_Position_Of_YZ_Plane+0.001,0,0));
		Initial_Slice_Location=CoorSystem->getRot()*Initial_Slice_Location;
		if(strcmp(SubObject->m_objectName,"YZ_Plane_1")==0){
			SubObject->setPos(Initial_Slice_Location);
//			YZ_Texture->loadFromFile("./Texture/COKE.bmp");
//			SubObject->setTexture(YZ_Texture,0);
//			SubObject->useTexture(1,0);
		}

		Initial_Slice_Location.copyfrom(cVector3d(Initial_Position_Of_YZ_Plane-0.001,0,0));
		Initial_Slice_Location=CoorSystem->getRot()*Initial_Slice_Location;
		if(strcmp(SubObject->m_objectName,"YZ_Plane_2")==0){
			SubObject->setPos(Initial_Slice_Location);
//			SubObject->setTexture(YZ_Texture,0);
//			SubObject->useTexture(1,0);
		}

		printf("CoorSystem Clild: %d, Name: %s\n",Sub_Object_Number,SubObject->m_objectName);
		printf("Arxiki Thesi: %f, %f, %f\n\n",SubObject->getPos().x,SubObject->getPos().y,SubObject->getPos().z);
	}
	CoorSystem->computeGlobalPositions(true);

    // create a torus like shape
    TorusObject = new cShapeTorus(0.14, 0.14);
    world->addChild(TorusObject);
    TorusObject->translate(0,1.8,0);
    TorusObject->m_material.setStiffness(20.0);
    TorusObject->m_material.setDynamicFriction(0.0);
    TorusObject->m_material.setStaticFriction(0.0);
//	TorusObject->setFrameSize(0.5,1,false);
//	TorusObject->setShowFrame(true,true);
    TorusObject->m_texture = NULL;
     // let's create a some environment mapping
/*	TorusObjectTexture = new cTexture2D();
	TorusObjectTexture->loadFromFile("./Texture/TorusObject.bmp");
	TorusObjectTexture->setEnvironmentMode(GL_DECAL);
	TorusObjectTexture->setSphericalMappingEnabled(true);
    TorusObject->m_texture = TorusObjectTexture;
*/
    ShapeSphere_Delete_Object = new cShapeSphere(0.14);
    world->addChild(ShapeSphere_Delete_Object);
    ShapeSphere_Delete_Object->translate(0,1.8,0.8);
    ShapeSphere_Delete_Object->m_material.setStiffness(20.0);
    ShapeSphere_Delete_Object->m_material.setDynamicFriction(0.0);
    ShapeSphere_Delete_Object->m_material.setStaticFriction(0.0);
//	ShapeSphere_Delete_Object->setFrameSize(0.5,1,false);
//	ShapeSphere_Delete_Object->setShowFrame(true,true);
    ShapeSphere_Delete_Object->m_texture = NULL;
    // let's create a some environment mapping
/*	ShapeSphere_Delete_Object_Texture = new cTexture2D();
	ShapeSphere_Delete_Object_Texture->loadFromFile("./Texture/ShapeSphere_Delete_Object_Texture.bmp");
	ShapeSphere_Delete_Object_Texture->setEnvironmentMode(GL_DECAL);
	ShapeSphere_Delete_Object_Texture->setSphericalMappingEnabled(true);
    ShapeSphere_Delete_Object->m_texture = ShapeSphere_Delete_Object_Texture;
*/
    ShapeSphere_Add_Object = new cShapeSphere(0.14);
    world->addChild(ShapeSphere_Add_Object);
    ShapeSphere_Add_Object->translate(0,1.8,-0.8);
    ShapeSphere_Add_Object->m_material.setStiffness(20.0);
    ShapeSphere_Add_Object->m_material.setDynamicFriction(0.0);
    ShapeSphere_Add_Object->m_material.setStaticFriction(0.0);
//	ShapeSphere_Add_Object->setFrameSize(0.5,1,false);
//	ShapeSphere_Add_Object->setShowFrame(true,true);
    ShapeSphere_Add_Object->m_texture =NULL;
     // let's create a some environment mapping
/*	ShapeSphere_Add_Object_Texture = new cTexture2D();
	ShapeSphere_Add_Object_Texture->loadFromFile("./Texture/ShapeSphere_Add_Object.bmp");
	ShapeSphere_Add_Object_Texture->setEnvironmentMode(GL_DECAL);
	ShapeSphere_Add_Object_Texture->setSphericalMappingEnabled(true);
    ShapeSphere_Add_Object->m_texture =ShapeSphere_Add_Object_Texture;
*/
    Increase_ROI_Size_Object = new cShapeSphere(0.14);
    world->addChild(Increase_ROI_Size_Object);
    Increase_ROI_Size_Object->translate(0,-1.8,0.8);
    Increase_ROI_Size_Object->m_material.setStiffness(20.0);
    Increase_ROI_Size_Object->m_material.setDynamicFriction(0.0);
    Increase_ROI_Size_Object->m_material.setStaticFriction(0.0);
//	Increase_ROI_Size_Object->setFrameSize(0.5,1,false);
//	Increase_ROI_Size_Object->setShowFrame(true,true);
    Increase_ROI_Size_Object->m_texture = NULL;
    // let's create a some environment mapping
/*	Increase_ROI_Size_Texture = new cTexture2D();
	Increase_ROI_Size_Texture->loadFromFile("./Texture/Increase_ROI_Size_Texture.bmp");
	Increase_ROI_Size_Texture->setEnvironmentMode(GL_DECAL);
	Increase_ROI_Size_Texture->setSphericalMappingEnabled(true);
    Increase_ROI_Size_Object->m_texture = Increase_ROI_Size_Texture;
*/
    Decrease_ROI_Size_Object = new cShapeSphere(0.14);
    world->addChild(Decrease_ROI_Size_Object);
    Decrease_ROI_Size_Object->translate(0,-1.8,-0.8);
    Decrease_ROI_Size_Object->m_material.setStiffness(20.0);
    Decrease_ROI_Size_Object->m_material.setDynamicFriction(0.0);
    Decrease_ROI_Size_Object->m_material.setStaticFriction(0.0);
//	Decrease_ROI_Size_Object->setFrameSize(0.5,1,false);
//	Decrease_ROI_Size_Object->setShowFrame(true,true);
    Decrease_ROI_Size_Object->m_texture = NULL;
    // let's create a some environment mapping
/*	Decrease_ROI_Size_Texture = new cTexture2D();
	Decrease_ROI_Size_Texture->loadFromFile("./Texture/Decrease_ROI_Size_Texture.bmp");
	Decrease_ROI_Size_Texture->setEnvironmentMode(GL_DECAL);
	Decrease_ROI_Size_Texture->setSphericalMappingEnabled(true);
    Decrease_ROI_Size_Object->m_texture = Decrease_ROI_Size_Texture;
*/	//==================================================

    // initialize the object's rotational velocity with a slight motion
    rotVelocity.set(0.0, 0.0, 0.0);


    // create a tool and add it to the world.
	cursor=new cMeta3dofPointer(world, 0, USE_PHANTOM_DIRECT_IO);
	cursor->setRenderingMode(RENDER_PROXY);
			
    world->addChild(cursor);
    cursor->setPos(1.0, 0.0, 0.0);

    // set up a nice-looking workspace for the tool so it fits nicely with our
    // cube models we will be building
    cursor->setWorkspace(10.0,10.0,10.0);

    // set the diameter of the ball representing the tool
    cursor->setRadius(0.001);

	// set up the device
	cursor->initialize();

	// open communication to the device
	cursor->start();
		
	// update initial orientation and position of device
	cursor->updatePose();
		
	// tell the tool to show his coordinate frame so you
	// can see tool rotation
	cursor->visualizeFrames(true);
		
	// I need to call this so the tool can update its internal
	// transformations before performing collision detection, etc.
	cursor->computeGlobalPositions();
	cursor->setForcesON();
	
    // Make sure the haptic device knows where he is in the camera frame
    world->computeGlobalPositions(true);

    // start haptic timer callback
    // timer.set(0, hapticsLoop, NULL);

    // initialize the GLUT windows
    //glutInit(&argc, argv);
    //glutInitWindowSize(800,800);
    //glutInitWindowPosition(0, 0);
    //glutInitDisplayMode(GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE);
    //glutCreateWindow(argv[0]);
    //glutDisplayFunc(draw);
    //glutKeyboardFunc(Keyboard);
    //glutReshapeFunc(rezizeWindow);
    //glutSetWindowTitle("Multi-Approach DT-MRI Data Analysis & Visualization Platform");

    //// create a mouse menu
    //glutCreateMenu(setOther);
    //glutAddMenuEntry("Full Screen", OPTION_FULLSCREEN);
    //glutAddMenuEntry("Window Display", OPTION_WINDOWDISPLAY);
    //glutAddMenuEntry("New ROI", OPTION_INSERT_NEW_ROI);
    //glutAddMenuEntry("Re-start Haptic Device", RESTART_HAPTIC);
    //glutAddMenuEntry("Messages", MESSAGES);
    //glutAttachMenu(GLUT_RIGHT_BUTTON);


    // update display
//    glutTimerFunc(50, updateDisplay, 0);

    // start main graphic rendering loop
 //   glutMainLoop();



	// INIT QT
	QApplication app(argc, argv);

	// define the container-widget
	// ContainerWidget *container = new ContainerWidget;

	// open the QGL-widget
	GlWidget = new Simple;
	GlWidget->camera = camera;
	GlWidget->show();
	GlWidget->Initialize(cursor, CoorSystem, &Milliseconds_Rate_Update,&ROI_Scale_Factor,&CoorSystem_Rotation_Factor);
	GlWidget->resize(800, 800);

	// initialize the haptic messenger
	HapticMessenger = new vavHapticMessenger;
	HapticMessenger->HapticLoopfunction = hapticsLoop;

	// define a simple QVTK-widget
	SimpleQVTK *sqvtk = new SimpleQVTK();
	sqvtk->show();



	// establish connections
	//QObject::connect( HapticMessenger, SIGNAL(Test()),
	//sqvtk, SLOT(Test()) );

	QObject::connect( HapticMessenger, SIGNAL(ChangeSlicePosition(double, double, double)),
					  sqvtk, SLOT(ChangeSlicePosition(double, double, double)) );

	QObject::connect( HapticMessenger, SIGNAL(DeleteROI(char*)),
					  sqvtk, SLOT(DeleteROI(char*)) );

	//QObject::connect( HapticMessenger, SIGNAL(SceneRotated(cMatrix3d)),
	//				  sqvtk, SLOT(SceneRotated(cMatrix3d)) );

	QObject::connect( HapticMessenger, SIGNAL(SceneRotated2(cVector3d, double)),
					  sqvtk, SLOT(SceneRotated2(cVector3d, double)) );


	QObject::connect( HapticMessenger, SIGNAL( AddROI(char*, cVector3d, double) ),
					  sqvtk, SLOT( AddROI(char*, cVector3d, double) ) );

	QObject::connect( HapticMessenger, SIGNAL( UpdateHapticCursor(cVector3d) ),
					  sqvtk, SLOT( UpdateHapticCursor(cVector3d) ) );

	QObject::connect( HapticMessenger, SIGNAL( TransfromROI(char*, cVector3d, cMatrix3d) ),
					  sqvtk, SLOT( TransfromROI(char*, cVector3d, cMatrix3d) ) );

	QObject::connect( HapticMessenger, SIGNAL( DeleteROI(char*) ),
					  sqvtk, SLOT( DeleteROI(char*) ) );

	QObject::connect( HapticMessenger, SIGNAL( ScaleROI(char*,double) ),
					  sqvtk, SLOT( ScaleROI(char*,double) ) );


	// set up the device
	cursor->initialize();

	// open communication to the device
	cursor->start();

	// start haptic timer callback
//	timer.set(0, hapticsLoop, NULL);

	return app.exec();
//    return 0;
}

//---------------------------------------------------------------------------
