

#include "simple.h"
#include "CCamera.h"

#include <QtGui>
#include <QtOpenGL>
#include <QTimer>
#include "glut.h"
#include "vtkMath.h"
#include "cMeta3dofPointer.h"
#include "CMesh.h"



Simple::Simple(QWidget *parent) : QGLWidget(parent)
{
	setFormat( QGLFormat(QGL::DoubleBuffer | QGL::DepthBuffer) );
	camera = NULL;
	cursor = NULL;

	// register the thread and start it!
	//updateDisplayThread = new UpdateThread(this);
	//updateDisplayThread->simpleGLWidget = this;

	// connect the thread to this class
	// connect( updateDisplayThread, SIGNAL(UpdateView()), this, SLOT(UpdateView()) );

	//updateDisplayThread->start();

	//if( updateDisplayThread->isRunning() )
	//	printf( "thread is running now...\n" );

	QTimer *timer = new QTimer(this);
	connect(timer, SIGNAL(timeout()), this, SLOT(UpdateView()));
	timer->start(20);
}


void Simple::Initialize( cMeta3dofPointer *_cursor, cMesh *_CoorSystem, int *_Milliseconds_Rate_Update,double *_ROI_Scale_Factor,double *_CoorSystem_Rotation_Factor)
{
	cursor = _cursor;
	CoorSystem = _CoorSystem;
	Milliseconds_Rate_Update = _Milliseconds_Rate_Update;
	HapticCursorPosition[0] = cursor->m_deviceGlobalPos.x;
	HapticCursorPosition[1] = cursor->m_deviceGlobalPos.y;
	HapticCursorPosition[2] = cursor->m_deviceGlobalPos.z;
	ROI_Scale_Factor=_ROI_Scale_Factor;
	CoorSystem_Rotation_Factor=_CoorSystem_Rotation_Factor;
}


void Simple::Redraw()
{
	this->updateGL();
}


void Simple::initializeGL()
{
	qglClearColor(Qt::black);
	glShadeModel(GL_SMOOTH);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
}

void Simple::resizeGL(int w, int h)
{
	width = w;
	height = h;
	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	GLfloat x = GLfloat(width) / height;
	glFrustum(-x, x, -1.0, 1.0, 4.0, 15.0);
	glMatrixMode(GL_MODELVIEW);
}


// ----------------------------------------------------------------------
void Simple::paintGL()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	if( camera != NULL )
		camera->renderView(width, height);	

	// QGLWidget::swapBuffers();
}


// ----------------------------------------------------------------------
void Simple::mousePressEvent(QMouseEvent *event)
{
}


// ----------------------------------------------------------------------
void Simple::mouseMoveEvent(QMouseEvent *event)
{
}


//----------------------------------------------------------------------
void Simple::mouseDoubleClickEvent(QMouseEvent *event)
{
	//
}


//----------------------------------------------------------------------
void Simple::keyPressEvent(QKeyEvent *event)
{
	if( CoorSystem != NULL )
	{
		if(event->key() == Qt::Key_Z ) CoorSystem->setHapticEnabled(false,true);
		if(event->key() == Qt::Key_X ) CoorSystem->setHapticEnabled(true,true);

		if(event->key() == Qt::Key_C ){
			for (Object_Number = 0; Object_Number<CoorSystem->getNumChildren(); Object_Number++){
				CurObject=(cMesh*)CoorSystem->getChild(Object_Number);
				if(strstr(CurObject->m_objectName,"_Plane")!=NULL){ 
					CurObject->enableTransparency(true, false);
					CurObject->setTransparencyLevel(0.6,false,false);
				}
			}
		}

		if(event->key() == Qt::Key_V ){
			for (Object_Number = 0; Object_Number<CoorSystem->getNumChildren(); Object_Number++){
				CurObject=(cMesh*)CoorSystem->getChild(Object_Number);
				if(strstr(CurObject->m_objectName,"_Plane")!=NULL){ 
					CurObject->enableTransparency(false, false);
				}
			}
		}
	}

	if( Milliseconds_Rate_Update != NULL )
	{
		if(event->key() == Qt::Key_W ) *Milliseconds_Rate_Update+=50;
		if(event->key() == Qt::Key_Q )
			if(*Milliseconds_Rate_Update>50) *Milliseconds_Rate_Update-=50;

	}

	if(event->key() == Qt::Key_B ){
		*ROI_Scale_Factor+=0.001;
		cout << "ROI_Scale_Factor = " << *ROI_Scale_Factor << endl;
	}
	if(event->key() == Qt::Key_N ){
		cout << "ROI_Scale_Factor = " << *ROI_Scale_Factor << endl;
		if(*ROI_Scale_Factor<1 && *ROI_Scale_Factor>0){
			*ROI_Scale_Factor-=0.001;
			cout << "ROI_Scale_Factor = " << *ROI_Scale_Factor << endl;
		}
	}

	this->updateGL();
}



//----------------------------------------------------------------------
void Simple::UpdateView()
{
	// if any change in the haptic cursor position is detected -> emit a signal
	if( cursor != NULL )
	{
		double newPosition[3];
		newPosition[0] = cursor->m_deviceGlobalPos.x;
		newPosition[1] = cursor->m_deviceGlobalPos.y;
		newPosition[2] = cursor->m_deviceGlobalPos.z;

		if( vtkMath::Distance2BetweenPoints(HapticCursorPosition, newPosition) > 1e-3 )
		{
			for( int i=0; i<3; i++ )
				HapticCursorPosition[i] = newPosition[i];

			emit HapticCursorPositionChanged(HapticCursorPosition);
		}
	}
	
	
	this->Redraw();
	// updateDisplayThread->start(); // restart the thread
}


//////////////////////////////////////////////////////////////////////////
//UpdateThread::UpdateThread(QWidget *parent) : QThread(parent)
//{
//
//}
//
//void UpdateThread::run()
//{
//	// emit UpdateView();
//	while(true)
//	{
//		msleep(1);
//		emit UpdateView();
//	}
//}