 
#include "vav3DViewer.h"


#include "vtkActor.h"
#include "vtkBoxWidget.h"
#include "vtkCamera.h"
#include "vtkCommand.h"
#include "vtkImageData.h"
#include "vtkImagePlaneWidget.h"
#include "vtkOutlineFilter.h"
#include "vtkPlane.h"
#include "vtkPointData.h"
#include "vtkPolyDataMapper.h"
#include "vtkProperty.h"
#include "vtkRenderer.h"
#include "vtkRenderWindow.h"
#include "vtkRenderWindowInteractor.h"
#include "vtkVolume16Reader.h"



//--------------------------------------------------------------------------------
class vavImagePlaneWidgetCallback : public vtkCommand
{
public:
	vav3DViewer *Viewer;

	static vavImagePlaneWidgetCallback* New()
	{ return new vavImagePlaneWidgetCallback; };

	virtual void Execute(vtkObject *caller, unsigned long eventId, void *callData)
	{
		vtkImagePlaneWidget *widget = (vtkImagePlaneWidget*) caller;
		Viewer->HandleImagePlaneWidgetEvent(0);
	}
};



//--------------------------------------------------------------------------------
vav3DViewer::vav3DViewer(QWidget *parent) : QVTKWidget(parent)
{
	// read data
	v16 =  vtkVolume16Reader::New();
	v16->SetDataDimensions( 64, 64);
	v16->SetDataByteOrderToLittleEndian();
	v16->SetImageRange( 1, 93);
	v16->SetDataSpacing( 3.2, 3.2, 1.5);
	v16->SetFilePrefix("C:/VTK/VTKData/Data/headsq/quarter");
	v16->SetDataMask(0x7fff);
	v16->Update();
	ImageData = v16->GetOutput();

	// create an instance of vav-scalar 
	//ScalarData = new VavScalar;
	//ScalarData->setImageArray(ImageData);

	Renderer = vtkRenderer::New();
	// RenderWindow = vtkRenderWindow::New();
	// this->SetRenderWindow(RenderWindow);
	RenderWindow = GetRenderWindow();
	RenderWindow->AddRenderer(Renderer);
	RenderWindowInteractor = this->GetInteractor();
	

	outline = vtkOutlineFilter::New();
	outline->SetInputConnection(v16->GetOutputPort());
	outlineMapper = vtkPolyDataMapper::New();
	outlineMapper->SetInputConnection(outline->GetOutputPort());
	outlineActor =  vtkActor::New();
	outlineActor->SetMapper(outlineMapper);
	outlineActor->GetProperty()->SetColor(0,0,0);

	InitImagePlaneWidgets();

	// get scalar range of the data
	double range[2];
	ImageData->GetScalarRange(range);


	// BOX-WIDGET
	BoxWidget = vtkBoxWidget::New();
	BoxWidget->SetInteractor(this->GetInteractor());
	double bounds[6];
	ImageData->GetBounds(bounds);
	BoxWidget->PlaceWidget(bounds);
	BoxWidget->GetOutlineProperty()->SetColor(0, 1, 0);
	BoxWidget->GetSelectedOutlineProperty()->SetColor(0, 1, 0);
	BoxWidget->On();

	// init renderer
	Renderer->SetBackground( 1, 1, 1);
	Renderer->AddActor(outlineActor);
	// Renderer->AddActor(extrusionActor);
}


//--------------------------------------------------------------------------------
void vav3DViewer::InitImagePlaneWidgets()
{
	int midSliceIndex[] = { (ImageData->GetExtent()[1] - ImageData->GetExtent()[0])/2,
							(ImageData->GetExtent()[3] - ImageData->GetExtent()[2])/2,
							(ImageData->GetExtent()[5] - ImageData->GetExtent()[4])/2 };


	// SAGGITAL
	SaggitalImagePlaneWidget = vtkImagePlaneWidget::New();
	SaggitalImagePlaneWidget->TextureInterpolateOff();
	SaggitalImagePlaneWidget->UserControlledLookupTableOff();
	SaggitalImagePlaneWidget->SetInput(v16->GetOutput());
	SaggitalImagePlaneWidget->KeyPressActivationOn();
	SaggitalImagePlaneWidget->SetKeyPressActivationValue('x');
	SaggitalImagePlaneWidget->SetResliceInterpolateToNearestNeighbour();
	SaggitalImagePlaneWidget->SetInteractor(RenderWindowInteractor);
	SaggitalImagePlaneWidget->SetPlaneOrientationToXAxes();
	SaggitalImagePlaneWidget->SetSliceIndex( midSliceIndex[0] );
	SaggitalImagePlaneWidget->GetPlaneProperty()->SetColor(1,0,0);
	SaggitalImagePlaneWidget->On();
	SaggitalImagePlaneWidget->SetInteraction(0);
	SaggitalImagePlaneWidget->UpdatePlacement();

	// CORONAL
	CoronalImagePlaneWidget = vtkImagePlaneWidget::New();
	CoronalImagePlaneWidget->TextureInterpolateOff();
	CoronalImagePlaneWidget->UserControlledLookupTableOff();
	CoronalImagePlaneWidget->SetInput(v16->GetOutput());
	CoronalImagePlaneWidget->KeyPressActivationOn();
	CoronalImagePlaneWidget->SetKeyPressActivationValue('y');
	CoronalImagePlaneWidget->SetResliceInterpolateToNearestNeighbour();
	CoronalImagePlaneWidget->SetInteractor(RenderWindowInteractor);
	CoronalImagePlaneWidget->SetPlaneOrientationToYAxes();
	CoronalImagePlaneWidget->SetSliceIndex( midSliceIndex[1] );
	CoronalImagePlaneWidget->GetPlaneProperty()->SetColor(0,1,0);
	CoronalImagePlaneWidget->On();
	CoronalImagePlaneWidget->SetInteraction(0);
	CoronalImagePlaneWidget->UpdatePlacement();


	// AXIAL
	AxialImagePlaneWidget = vtkImagePlaneWidget::New();
	AxialImagePlaneWidget->TextureInterpolateOff();
	AxialImagePlaneWidget->UserControlledLookupTableOff();
	AxialImagePlaneWidget->SetInput(v16->GetOutput());
	AxialImagePlaneWidget->KeyPressActivationOn();
	AxialImagePlaneWidget->SetKeyPressActivationValue('z');
	AxialImagePlaneWidget->SetResliceInterpolateToNearestNeighbour();
	AxialImagePlaneWidget->SetInteractor(RenderWindowInteractor);
	AxialImagePlaneWidget->SetPlaneOrientationToZAxes();
	AxialImagePlaneWidget->SetSliceIndex( midSliceIndex[2] );
	AxialImagePlaneWidget->GetPlaneProperty()->SetColor(0,0,1);
	AxialImagePlaneWidget->On();
	AxialImagePlaneWidget->SetInteraction(0);
	AxialImagePlaneWidget->UpdatePlacement();

	// OBLIQUE
	ObliqueImagePlaneWidget = vtkImagePlaneWidget::New();
	ObliqueImagePlaneWidget->TextureInterpolateOff();
	ObliqueImagePlaneWidget->UserControlledLookupTableOff();
	ObliqueImagePlaneWidget->SetInput(v16->GetOutput());
	ObliqueImagePlaneWidget->KeyPressActivationOn();
	ObliqueImagePlaneWidget->SetKeyPressActivationValue('o');
	ObliqueImagePlaneWidget->SetResliceInterpolateToNearestNeighbour();
	ObliqueImagePlaneWidget->SetInteractor(RenderWindowInteractor);
	ObliqueImagePlaneWidget->SetPlaneOrientationToXAxes();
	ObliqueImagePlaneWidget->SetSliceIndex(32);
	ObliqueImagePlaneWidget->GetPlaneProperty()->SetColor(1,1,0);

	// add a callback to the IPW
	ImagePlaneWidgetCallback = vavImagePlaneWidgetCallback::New();
	ImagePlaneWidgetCallback->Viewer = this;
	ObliqueImagePlaneWidget->AddObserver(vtkCommand::InteractionEvent, ImagePlaneWidgetCallback);

	// enable the oblique image-plane-widget
	ObliqueImagePlaneWidget->On();
	ObliqueImagePlaneWidget->SetInteraction(0);
	ObliqueImagePlaneWidget->SetInteraction(1);
}


//--------------------------------------------------------------------------------
void vav3DViewer::HandleImagePlaneWidgetEvent( int eventID )
{
	// Slice Changed
	if( eventID == 0 )
		emit ObliqueSliceChanged( ObliqueImagePlaneWidget->GetResliceOutput() );

	// One of possible other events would be changing the Window/Level of the
	// oblique-slicer which needs to be reflected to the other slicers in the 3d-scene:
	// in such a case, use another event-id to distinguish it from the other event(s).
}


//--------------------------------------------------------------------------------
vtkImageData* vav3DViewer::GetSaggitalSlice()
{ return SaggitalImagePlaneWidget->GetResliceOutput(); }


//--------------------------------------------------------------------------------
vtkImageData* vav3DViewer::GetCoronalSlice()
{ return CoronalImagePlaneWidget->GetResliceOutput(); }


//--------------------------------------------------------------------------------
vtkImageData* vav3DViewer::GetAxialSlice()
{ return AxialImagePlaneWidget->GetResliceOutput(); }


//--------------------------------------------------------------------------------
void vav3DViewer::SetSaggitalSliceIndex(int index)
{
	SaggitalImagePlaneWidget->SetSliceIndex(index);
	SaggitalImagePlaneWidget->UpdatePlacement();
	UpdateView();
	emit SaggitalSliceChanged( GetSaggitalSlice() );
}


//--------------------------------------------------------------------------------
void vav3DViewer::SetCoronalSliceIndex(int index)
{
	CoronalImagePlaneWidget->SetSliceIndex(index);
	CoronalImagePlaneWidget->UpdatePlacement();
	UpdateView();
	emit CoronalSliceChanged( GetCoronalSlice() );
}


//--------------------------------------------------------------------------------
void vav3DViewer::SetAxialSliceIndex(int index)
{
	AxialImagePlaneWidget->SetSliceIndex(index);
	AxialImagePlaneWidget->UpdatePlacement();
	UpdateView();
	emit AxialSliceChanged( GetAxialSlice() );
}


//--------------------------------------------------------------------------------
void vav3DViewer::UpdateView()
{ this->GetRenderWindow()->Render(); }











