
#ifndef _vavHapticMessenger_h
#define _vavHapticMessenger_h


#include <QObject>
#include "CVector3d.h"
#include "CMatrix3d.h"

class CVector3d;
class CMatrix3d;


class vavHapticMessenger : public QObject
{
	Q_OBJECT
public:
	vavHapticMessenger( QWidget *parent=0 );

	// functions for making this object emit signals
	void Emit_DeleteROI_Signal(char *roiName);
	void Emit_AddROI_Signal(char *roiName, cVector3d roiPosition, double radius);
	void Emit_TransfromROI_Signal(char *roiName, cVector3d roiPosition, cMatrix3d roiRotation);
	void Emit_ScaleROI_Signal(double scaleFactor);

	// emitted when the position of the slices are changed
	void Emit_ChangeSlicePosition_Signal(cVector3d);

	// emitted when the scene is rotated
	void Emit_SceneRotated_Signal(cMatrix3d VtK_Rotation_Global);
	void Emit_SceneRotated2_Signal(cVector3d axis, double rotAngle);

	void Emit_UpdateHapticCursor_Signal(cVector3d);

	void Emit_ScaleROI_Signal(char*, double);

	// Pointer to the Haptic-Loop function called every X MSECs
	void (*HapticLoopfunction)(void*) ;


private slots:
	void Update();

// SIGNALS
signals:
	// ROI-SIGNALS
	void DeleteROI(char *roiName);
	void AddROI(char*, cVector3d, double);
	void TransfromROI(char*, cVector3d, cMatrix3d);
	void ScaleROI(char*, double);

	// SLICE-SIGNAL
	void ChangeSlicePosition(double, double, double);

	// SCENE-ROTATED-SIGNAL
	void SceneRotated(cMatrix3d VtK_Rotation_Global);
	void SceneRotated2(cVector3d axis, double rotAngle);

	///
	void UpdateHapticCursor(cVector3d);

	void Test();

	// WORLD-ROTATION-SIGNALS
};

#endif